CREATE OR REPLACE PROCEDURE MT_MODEL_DELETE(MODELID   IN NUMBER  DEFAULT NULL,
                                                             REFERENCE 	IN NVARCHAR2  DEFAULT NULL)
AS
MODELID_     NUMBER(10) := MODELID;
REFERENCE_   NVARCHAR2(64) := REFERENCE;
STOO_SELCNT 		INTEGER;
STOO_ERROR 			INTEGER;
STOO_ROWCNT		INTEGER;
STOO_CROWCNT     INTEGER := 0;
STOO_FETCHSTATUS INTEGER := 0;
STOO_ERRMSG			VARCHAR2(255);
STOO_SQLSTATUS	INTEGER;

SQLQUERY      		VARCHAR2(4096);         
NAME_					VARCHAR2(256);
TYPE_					VARCHAR2(256);
MODELPREFIX			VARCHAR2(256) := '%M' || TO_CHAR(MT_MODEL_DELETE.MODELID_) || '\_%';
MODELREFPREFIX			VARCHAR2(256) := UPPER(MT_MODEL_DELETE.REFERENCE_) || '\_%';

CURSOR       OBJECTS_LIST IS 
             SELECT 	DISTINCT OBJECT_NAME, OBJECT_TYPE
             FROM 		USER_OBJECTS
             WHERE  	(OBJECT_TYPE IN ('TABLE', 'VIEW', 'PROCEDURE', 'SEQUENCE') AND OBJECT_NAME LIKE MT_MODEL_DELETE.MODELPREFIX ESCAPE '\') OR 
                        (OBJECT_TYPE='VIEW' AND OBJECT_NAME LIKE MT_MODEL_DELETE.MODELREFPREFIX ESCAPE '\')
             ORDER 	BY OBJECT_TYPE;

BEGIN
	IF  MT_MODEL_DELETE.MODELID_ = 0 THEN
		BEGIN
			RAISE_APPLICATION_ERROR(-20999, 'OROSMASTERMODEL SHOULD NOT BE DELETED.  PLEASE CONTACT THE ADMINISTRATOR.');
			GOTO ON_ERROR;
		END;
	END IF;
	/* EMULATING @@TRANCOUNT FUNCTIONALITY IN ORACLE MODEL */
	ABM_PKG_GLOBALS.TRANCOUNT:=ABM_PKG_GLOBALS.TRANCOUNT+1;
	OPEN OBJECTS_LIST;
	LOOP
		FETCH OBJECTS_LIST INTO MT_MODEL_DELETE.NAME_, MT_MODEL_DELETE.TYPE_;
		EXIT WHEN OBJECTS_LIST%NOTFOUND;
		BEGIN
			MT_MODEL_DELETE.SQLQUERY := 'DROP ';
			MT_MODEL_DELETE.SQLQUERY := MT_MODEL_DELETE.SQLQUERY || MT_MODEL_DELETE.TYPE_ || ' ' || MT_MODEL_DELETE.NAME_;
			IF(MT_MODEL_DELETE.TYPE_ = 'TABLE') THEN
				MT_MODEL_DELETE.SQLQUERY := 'LOCK TABLE ' || ' ' || MT_MODEL_DELETE.NAME_ || ' IN EXCLUSIVE MODE';
				EXECUTE IMMEDIATE MT_MODEL_DELETE.SQLQUERY;
				MT_MODEL_DELETE.SQLQUERY := 'DROP TABLE ' || ' ' || MT_MODEL_DELETE.NAME_  || ' CASCADE CONSTRAINTS';
				EXECUTE IMMEDIATE MT_MODEL_DELETE.SQLQUERY;
				COMMIT;
			ELSE
				EXECUTE IMMEDIATE(MT_MODEL_DELETE.SQLQUERY);
			END IF;
			EXCEPTION
				WHEN NO_DATA_FOUND THEN
					    NULL;
				WHEN OTHERS THEN
					    STOO_ERROR := SQLCODE;
					    STOO_ERRMSG := SQLERRM;
					    RAISE_APPLICATION_ERROR(-20000, SQLERRM,TRUE); 
		END;
	END LOOP;
	CLOSE OBJECTS_LIST;
	BEGIN
		STOO_ERROR   := 0;
		STOO_ROWCNT  := 0;
		DELETE  MODELPERIODSCENARIO 
		WHERE	MODELID = MT_MODEL_DELETE.MODELID_;
		STOO_ROWCNT := SQL%ROWCOUNT;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				    NULL;
			WHEN OTHERS THEN
				    STOO_ERROR := SQLCODE;
				    STOO_ERRMSG := SQLERRM;
				    RAISE_APPLICATION_ERROR(-20000, SQLERRM,TRUE); 
	END;

	/* EMULATING @@TRANCOUNT FUNCTIONALITY IN ORACLE MODEL */
	IF ABM_PKG_GLOBALS.TRANCOUNT > 0 THEN
		ABM_PKG_GLOBALS.TRANCOUNT:=ABM_PKG_GLOBALS.TRANCOUNT-1;
	END IF;
	<<ON_ERROR>> NULL;
END MT_MODEL_DELETE;
/
